#!/usr/bin/python
# -*- coding: iso-8859-2 -*-

# 12.02.2007
# Mateusz Styrczula

""" 3D camera management """

import sys
import os
from OpenGL.GLUT import *
from OpenGL.GL import *
from OpenGL.GLU import *

from angle import *




class Camera:
    """ Virtual Camera class """

    def __init__ (self,
                  fovy = Angle (45),
                  aspect = 4.0 / 3.0,
                  zNear = 10.0, zFar = 2000.0,
                  eye = [0.0, 0.0, 200.0],
                  center = [0.0, 0.0, 0.0],
                  up = [0.0, 1.0, 0.0]):
        """ default constructor """
        
        # vertical field-of-view angle
        self.fovy = fovy
        # aspect ratio (width/height)
        self.aspect = aspect
        # clipping planes
        self.zNear = zNear
        self.zFar = zFar
        # camera coordinates
        self.eye = eye
        self.center = center
        self.up = up
        # camera rotation angles
        self.phi = Angle (0.0)
        self.theta = Angle (0.0)


    def establish (self):
        """ establish camera position in scene """
        glRotatef (self.theta.get_deg (), 1.0, 0.0, 0.0)
        glRotatef (self.phi.get_deg (), 0.0, 1.0, 0.0)


    def establish_projection (self):
        """ establish main parameters """
        glMatrixMode (GL_PROJECTION)
        glLoadIdentity ()
        gluPerspective (self.fovy.get_deg (),
                        self.aspect,
                        self.zNear,
                        self.zFar)
        gluLookAt (self.eye[0],
                   self.eye[1],
                   self.eye[2],
                   self.center[0],
                   self.center[1],
                   self.center[2],
                   self.up[0],
                   self.up[1],
                   self.up[2])
        glMatrixMode (GL_MODELVIEW)

