#!/usr/bin/python
# -*- coding: iso-8859-2 -*-

# 16.02.2007
# Mateusz Styrczula

""" material handling classes """

import sys
import os
from string import *
from OpenGL.GLUT import *
from OpenGL.GL import *
from OpenGL.GLU import *

from misc import *
from angle import *




class Material:
    """ material description """

    def __init__ (self):
        """ default constructor """
        self.name = None
        self.shader_type = 0
        self.wire = False
        self.two_sided = False
        self.faceted = False
        self.use_self_illum_color = False
        self.ambient = [0.0, 0.0, 0.0]
        self.diffuse = [0.0, 0.0, 0.0]
        self.specular = [0.0, 0.0, 0.0]
        self.specular_level = 0.0
        self.self_illum_amount = 0.0
        self.self_illum_color = [0.0, 0.0, 0.0]
        self.opacity = 1.0
        self.glossiness = 0.0




class MaterialLibrary:
    """ material library container and file parser """

    def __init__ (self):
        """ default constructor """
        self.materials = {}
        self.add_default_material ()


    def add_default_material (self):
        """ define and add to collection "default" material """
        m = Material ()
        m.name = "__default__"
        m.ambient = [0.1, 0.1, 0.1]
        m.diffuse = [0.5, 0.6, 0.7]
        m.specular = [0.7, 0.7, 0.7]
        m.specular_level = 0.5
        m.glossiness = 10.0
        self.add_material (m)


    def parse_file (self, filename):
        """ read material descriptions from file """
        try:
            # open file
            in_file = open (filename, 'r')
            # parse file
            while True:
                material_name = in_file.readline ()
                if material_name == "":
                    break
                # create new material to write to
                m = Material ()
                m.name = material_name[:-1]
                # read shader type
                m.shader_type = int (in_file.readline ())
                # read wire, two_sided, faceted and use_self_illum_color
                line = in_file.readline ()
                tmp = map (lambda x: bool (int (x)), split (line, ','))
                m.wire = tmp[0]
                m.two_sided = tmp[1]
                m.faceted = tmp[2]
                m.use_self_illum_color = tmp[3]
                # read (and convert from 0.0-255.0 to 0.0-1.0 scale) ambient color
                line = in_file.readline ()
                m.ambient = map (lambda x: convert_scale (float (x), 255.0, 1.0), split (line, ','))
                # read (and convert from 0.0-255.0 to 0.0-1.0 scale) diffuse color
                line = in_file.readline ()
                m.diffuse = map (lambda x: convert_scale (float (x), 255.0, 1.0), split (line, ','))
                # read (and convert from 0.0-255.0 to 0.0-1.0 scale) specular color
                line = in_file.readline ()
                m.specular = map (lambda x: convert_scale (float (x), 255.0, 1.0), split (line, ','))
                # read specular level
                m.specular_level = convert_scale (float (in_file.readline ()), 100.0, 1.0)
                # update specular (color) by specular level
                m.specular = map (lambda x: x*m.specular_level, m.specular)
                # read self illumination amount
                m.self_illum_amount = float (in_file.readline ())
                # read (and convert from 0.0-255.0 to 0.0-1.0 scale) self illumination color
                line = in_file.readline ()
                m.self_illum_color = map (lambda x: convert_scale (float (x), 255.0, 1.0), split (line, ','))
                # read opacity
                m.opacity = convert_scale (float (in_file.readline ()), 100.0, 1.0)
                # read glossiness
                m.glossiness = convert_scale (float (in_file.readline ()), 100.0, 128.0)
                # add just readed material
                self.add_material (m)
            # close file
            in_file.close ()
            #raise Exception ("debug")
        except Exception, arg:
            print ("%s: \"%s\"\n" % (arg, filename))
            sys.exit (-1)


    def add_material (self, m):
        """ add material to library """
        self.materials[m.name] = m


    def activate_material (self, name):
        """ activate material by its name """
        try:
            m = self.materials[name]
        except:
            m = self.materials["__default__"]
        glMaterialfv (GL_FRONT_AND_BACK, GL_AMBIENT, (m.ambient[0],
                                                      m.ambient[1],
                                                      m.ambient[2],
                                                      m.opacity))
        glMaterialfv (GL_FRONT_AND_BACK, GL_DIFFUSE, (m.diffuse[0],
                                                      m.diffuse[1],
                                                      m.diffuse[2],
                                                      m.opacity))
        glMaterialfv (GL_FRONT_AND_BACK, GL_SPECULAR, (m.specular[0],
                                                       m.specular[1],
                                                       m.specular[2],
                                                       m.opacity))
        glMaterialfv (GL_FRONT_AND_BACK, GL_EMISSION, (m.self_illum_color[0],
                                                       m.self_illum_color[1],
                                                       m.self_illum_color[2],
                                                       m.opacity))
        glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, m.glossiness)




