-- Geometry and Materials dumping script (MAX 7)
-- (creates two text files)
-- dumps objects hierarchy
-- Mateusz Styrczula
-- 23.02.2007

-- addModifier (Edit_Normals) will work only in modify mode (why??)
max modify mode


--  *** GEOMETRY DUMP ***
-- open file for geometry dump
out_name = ("c:/scene.geom")
out_file = createfile out_name
-- add unique ID to every object in the scene
for i = 1 to objects.count do (setUserProp objects[i] "__uid_" i)
-- find every object-roots in the scene (roots = rootNode.children)
roots = for o in objects where o.parent == undefined collect o

-- do some "dump" for every object in the scene
fn geometry_dump objs out_file = (
	if objs != undefined then (
		for obj in objs do (
			-- check if node is geometry-object
			if (not isKindOf obj GeometryClass) or
			   (classof obj == TargetObject) then
				continue
			-- resetting XForm is hardly "undoable", so better work on copy
			o = copy obj
			-- add all needed modifiers
			in coordsys parent o.rotation = Quat 0 0 0 1
			in coordsys parent o.scale = [1,1,1]
			resetXForm o
			addModifier o (Turn_To_Mesh ())
			addModifier o (Edit_Normals ())
			-- select object being dumped
			select o
			-- make reference to Edit_Normal modifier (from top of stack)
			oen = o.modifiers[1]
			oen.EditNormalsMod.SelLevel = #Object
			-- save amounts of normals, vertices and faces
			num_norms = oen.GetNumNormals ()
			num_verts = oen.GetNumVertices () 
			num_faces = oen.GetNumFaces ()
			-- store mesh name
			format "%\n" (obj.name) to:out_file
			-- store mesh unique id
			format "%\n" (getUserProp obj "__uid_") to:out_file
			-- store mesh parent unique id
			if obj.parent != undefined then
				format "%\n" (getUserProp obj.parent "__uid_") to:out_file
			else
				format "%\n" (0) to:out_file
			-- store mesh children unique ids
			obj_children = (for c in obj.children collect (getUserProp c "__uid_")) as string
			format "%\n" (substring obj_children 3 (obj_children.count - 3)) to:out_file
			in coordsys parent (
				-- store mesh position
				format "%,%,%\n" (obj.position.x) (obj.position.y) (obj.position.z) to:out_file
				-- store mesh rotation (convert quaternion representation to axis-angle)
				rot_axis = copy obj.rotation.axis
				format "%,%,%,%\n" (rot_axis.x) (rot_axis.y) (rot_axis.z) (obj.rotation.angle) to:out_file
				-- store mesh scale
				format "%,%,%\n" (obj.scale.x) (obj.scale.y) (obj.scale.z) to:out_file
			)
			-- store mesh material name
			try (
				format "%\n" (obj.material.name) to:out_file
			) catch (
				format "none\n" to:out_file
			)
			-- store mesh normals, vertices and faces amounts
			format "%,%,%\n" num_norms num_verts num_faces to:out_file
			oen.EditNormalsMod.SelLevel = #Normal
			-- store normals
			for i = 1 to num_norms do (
				norm = oen.GetNormal i
				format "%," norm to:out_file
			)
			format "\n" to:out_file
			oen.EditNormalsMod.SelLevel = #Object
			-- store verices
			for i = 1 to num_verts do (
				in coordsys local vert = getVert o i
				-- vert = ((getVert o i) - o.pivot) * (o.rotation) / o.scale
				-- vert = oen.getVertex i
				format "%," vert to:out_file
			)
			format "\n" to:out_file
			-- store faces (normal indexes)
			for fnr = 1 to num_faces do (
				face = [0,0,0]
				for i = 1 to oen.GetFaceDegree fnr do (
					face[i] = oen.GetNormalID fnr i
				)
				format "%," face to:out_file
			)
			format "\n" to:out_file
			-- store faces (vertex indexes)
			for fnr = 1 to num_faces do (
				face = [0,0,0]
				for i = 1 to oen.GetFaceDegree fnr do (
					face[i] = oen.GetVertexID fnr i
				)
				format "%," face to:out_file
			)
			format "\n" to:out_file
			-- delete copied object
			deselect o
			delete o
			-- recursively dump all children
			geometry_dump obj.children out_file
		) -- end for obj in objs
	) -- end if objs != undefined
) -- end fn geometry_dump objs

-- invoke function
geometry_dump roots out_file
-- deselect any objects that are currently selected
clearSelection ()
-- close geometry dump file
close out_file




--  *** MATERIALS DUMP ***
-- open file for materials dump
out_name = ("c:/scene.mats")
out_file = createfile out_name
-- do some "dump" for every material used in the scene
for m in sceneMaterials do (
	-- store material name
	format "%\n" (m.name) to:out_file
	-- store material shader type
	format "%\n" (m.shaderType) to:out_file
	-- store wire,twoSided,faceted,useSelfIllumColor
	try (
		if m.wire then wire = 1 else wire = 0
		if m.twoSided then twoSided = 1 else twoSided = 0
		if m.faceted then faceted = 1 else faceted = 0
		if m.useSelfIllumColor then usic = 1 else usic = 0
		format "%,%,%,%\n" wire twoSided faceted usic to:out_file
	) catch (
		format "0,0,0,0\n" to:out_file
	)
	-- store ambient color
	format "%,%,%\n" m.ambient.r m.ambient.g m.ambient.b to:out_file
	-- store diffuse color
	format "%,%,%\n" m.diffuse.r m.diffuse.g m.diffuse.b to:out_file
	-- store specular color
	format "%,%,%\n" m.specular.r m.specular.g m.specular.b to:out_file
	-- store specular level
	format "%\n" m.specularLevel to:out_file
	-- store selfIllumAmount
	format "%\n" m.selfIllumAmount to:out_file
	-- store selfIllum color
	format "%,%,%\n" m.selfIllumColor.r m.selfIllumColor.g m.selfIllumColor.b to:out_file
	-- store opacity
	format "%\n" m.opacity to:out_file
	-- store glossiness
	format "%\n" m.glossiness to:out_file
)
-- close materials dump file
close out_file
