#!/usr/bin/python
# -*- coding: iso-8859-2 -*-

# 12.02.2007
# Mateusz Styrczula

""" miscellaneous constants and functions """

import sys
import os
import math
from OpenGL.GLUT import *
from OpenGL.GL import *
from OpenGL.GLU import *




twopi = 2 * math.pi




def grid ():
    """ display grid """
    glBlendFunc (GL_SRC_ALPHA_SATURATE, GL_ONE_MINUS_SRC_ALPHA)
    glMaterialfv (GL_FRONT, GL_DIFFUSE, (0.0, 0.0, 0.0, 0.2))
    glMaterialfv (GL_FRONT, GL_SPECULAR, (0.0, 0.0, 0.0, 0.0))
    glPushMatrix ()
    glTranslatef (-60.0, 0.0, 0.0)
    for i in range (0, 11):
        glTranslatef (10.0, 0.0, 0.0)
        glBegin (GL_LINES)
        glVertex3f (0.0, 0.0, 0.0)
        glVertex3f (0.0, 0.0, 50.0)
        glEnd ()
        glBegin (GL_LINES)
        glVertex3f (0.0, 0.0, 0.0)
        glVertex3f (0.0, 0.0, -50.0)
        glEnd()
    glPopMatrix ()
    glPushMatrix ()
    glTranslatef (0.0, 0.0, -60.0)
    for i in range (0, 11):
        glTranslatef(0.0, 0.0, 10.0)
        glBegin (GL_LINES)
        glVertex3f (0.0, 0.0, 0.0)
        glVertex3f (50.0, 0.0, 0.0)
        glEnd ()
        glBegin (GL_LINES)
        glVertex3f (0.0, 0.0, 0.0)
        glVertex3f (-50.0, 0.0, 0.0)
        glEnd ()
    glPopMatrix ()
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA)




def cross_product (l, r):
    """ computes cross product of two 3D vectors """
    return [l[1] * r[2] - r[1] * l[2],
            l[2] * r[0] - r[2] * l[0],
            l[0] * r[1] - r[0] * l[1]]


def vector_length (v):
    """ computes vector length """
    return math.sqrt (reduce (lambda acc, x: acc + x**2, v[1:], v[0]**2))


def normalize (v):
    """ computes normalized vector """
    invlen = 1.0 / vector_length (v)
    return map (lambda x: x * invlen, v)


def vector_from_points (p, q):
    """ computes vector (3D) from two three-dimensional points """
    return [q[0] - p[0],
            q[1] - p[1],
            q[2] - p[2]]


def convert_scale (input, inmax, outmax):
    """ converts scale, eg. 0.0-255.0 -> 0.0-1.0 """
    return ((100.0 / inmax) * input) * 0.01 * outmax


